//SPECIFY THE Qi HERE:

q4 := -1;
q5 := 0;
q6 := 1;





//Rest is similar to AlgorithmGeneral.m.

//First we load some necessary algorithms.
//Copied from David Kohel's Echidna 2.0:

function DerivativeSequence(f,n)
    S := [ Parent(f) | ];
    for k in [0..n] do
        g := f;
        for i in [1..n] do
            if i le k then
                g := Derivative(g,1);
            else
                g := Derivative(g,2);
            end if;
        end for;
        S[k+1] := g; 
    end for;
    return S; 
end function;


function Transvectant(f,g,r)
    P := Parent(f);
    if f eq 0 or g eq 0 then return P!0; end if;
    Sf := DerivativeSequence(f,r);
    Sg := DerivativeSequence(g,r);
    Tfg := P!0;
    for k in [0..r] do
         Tfg +:= (-1)^k*Binomial(r,k)*Sf[k+1]*Sg[r-k+1];
    end for;
    n := TotalDegree(f); 
    m := TotalDegree(g);
    cfg := Factorial(n-r)*Factorial(m-r)/(Factorial(n)*Factorial(m));
    return cfg*Tfg;
end function;


function BinaryQuarticInvariants(bq);

R<x,y> := Parent(bq);

a := MonomialCoefficient(bq,x^4);
b := MonomialCoefficient(bq,x^3*y);
c := MonomialCoefficient(bq,x^2*y^2);
d := MonomialCoefficient(bq,x*y^3);
e := MonomialCoefficient(bq,y^4);

I := 12*a*e - 3*b*d + c^2;
J := 72*a*c*e + 9*b*c*d - 27*a*d^2 - 27*e*b^2 - 2*c^3;

return I,J;

end function;


//Creation of the Fuertes form  f  .

Q := Rationals();
RQ<t> := PolynomialRing(Q);

f3 := t^3 - 3*t + 1;
F<r> := SplittingField(f3);
Rts := Roots(f3,F);
r1 := Rts[1][1];
r2 := Rts[2][1];
r3 := Rts[3][1];
K := F;


R<x,y> := PolynomialRing(K,2);
S<t> := PolynomialRing(K);
g := hom<R -> S | [t,1]>;

f :=
 (x^4 - 2*(1 - 2*((r3 - r1)/(r3 - r2))*((q4 - r2)/(q4 - r1)))*x^2*y^2 + y^4)
*(x^4 - 2*(1 - 2*((r3 - r1)/(r3 - r2))*((q5 - r2)/(q5 - r1)))*x^2*y^2 + y^4)
*(x^4 - 2*(1 - 2*((r3 - r1)/(r3 - r2))*((q6 - r2)/(q6 - r1)))*x^2*y^2 + y^4)
;


//Defining a binary quartic covariant  T  of  f  :

T := Transvectant(f,f,10);
I,J := BinaryQuarticInvariants(T);
I := F!I;
J := F!J;

In := (I * (q4 - r)^4 * (q5 - r)^4 * (q6 - r)^4);
c := 1/14641*(7405568*r^2 - 13893632*r + 3932160);
test,rt := IsPower(c,2);
alpha := (1/rt) * (q4 - r)^2 * (q5 - r)^2 * (q6 - r)^2;

IQ := alpha^2 * I;
JQ := alpha^3 * J;

//We will use this relation later.
//I don't know if we are "lucky" because  alpha  is defined over the ground field.
//I suspect not, but let's do the descent first.


//Our descent method is extremely naive (so perhaps it could be improved!).
//We adjoin a root of the transvectant  T  so that this polynomial factors.
//Then over the corresponding field, we correspond the descent,
//by in fact reducing to binary cubics, which are somewhat simpler as regards isomorphisms.

L<a> := NumberField(g(T));

R<x,y> := PolynomialRing(L,2);
S<t> := PolynomialRing(L);
g := hom<R -> S | [t,1]>;


f :=
 (x^4 - 2*(1 - 2*((r3 - r1)/(r3 - r2))*((q4 - r2)/(q4 - r1)))*x^2*y^2 + y^4)
*(x^4 - 2*(1 - 2*((r3 - r1)/(r3 - r2))*((q5 - r2)/(q5 - r1)))*x^2*y^2 + y^4)
*(x^4 - 2*(1 - 2*((r3 - r1)/(r3 - r2))*((q6 - r2)/(q6 - r1)))*x^2*y^2 + y^4)
;

T := Transvectant(f,f,10);
I,J := BinaryQuarticInvariants(T);
I := L!I;
J := L!J;
j := 1728*(4*I^3)/(4*I^3 - J^2);

fac1 := (x - a*y);
test,fac2 := IsDivisibleBy(T,fac1);


//So we indeed have a factorization.
//Next, we construct a transformation of  x and y  under which  T  gets sent to
//the binary quartic  Tn2 = cL*y*(x^3 + aL*x*y^2 + bL*y^3)  .

M1 := Matrix(L,2,2,[1,-1,a,0]);
g1 := hom<R -> R | [M1[1,1]*x + M1[1,2]*y , M1[2,1]*x + M1[2,2]*y]>;
Tn1 := g1(T);
c := MonomialCoefficient(Tn1,x^2*y^2)/MonomialCoefficient(Tn1,x^3*y);
M2 := Matrix(L,2,2,[1,-c/3,0,1]);
g2 := hom<R -> R | [M2[1,1]*x + M2[1,2]*y , M2[2,1]*x + M2[2,2]*y]>;
Tn2 := g2(Tn1);

MonomialCoefficient(Tn2,x^4);
MonomialCoefficient(Tn2,x^2*y^2);

cL := MonomialCoefficient(Tn2,x^3*y);
detM := Determinant(M1*M2);


//We have
// -3 *cL^2*aL = I(Tn2) = detM^4*I(T) = (detM^4 / alpha^2)*IQ  ,
// -27*cL^3*bL = J(Tn2) = detM^6*J(T) = (detM^6 / alpha^3)*JQ  .

//So if we put  k = detM^2 / (cL * alpha)  , then
//under the transformation  x -> k*x , y -> y  we have
//that  Tn2  gets sent to
//  cL*y*(k^3*x^3 + aL*k*x*y^2 + bL*y^3)  
//= k^3*cL*y*(x^3 + (aL / k^2)*x*y^2 + (bL / k^3)*y^3)  
//= k^3*cL*y*(x^3 - (1/3)*IQ*x*y^2 - (1/27)*JQ*y^3)           .

M3 := Matrix(L,2,2,[(detM)^2/(cL*alpha),0,0,1]);
g3 := hom<R -> R | [M3[1,1]*x + M3[1,2]*y , M3[2,1]*x + M3[2,2]*y]>;
Tn3 := g3(Tn2);

//Test:
(-(1/3)*IQ*MonomialCoefficient(Tn3,x^3*y)) eq (MonomialCoefficient(Tn3,x*y^3));
(-(1/27)*JQ*MonomialCoefficient(Tn3,x^3*y)) eq (MonomialCoefficient(Tn3,y^4));

M := M1*M2*M3;

g := hom<R -> R | [M[1,1]*x + M[1,2]*y , M[2,1]*x + M[2,2]*y]>;

fac1 := (x^4 - 2*(1 - 2*((r3 - r1)/(r3 - r2))*((q4 - r2)/(q4 - r1)))*x^2*y^2 + y^4);
fac2 := (x^4 - 2*(1 - 2*((r3 - r1)/(r3 - r2))*((q5 - r2)/(q5 - r1)))*x^2*y^2 + y^4);
fac3 := (x^4 - 2*(1 - 2*((r3 - r1)/(r3 - r2))*((q6 - r2)/(q6 - r1)))*x^2*y^2 + y^4);

desc1 := g(fac1);
desc2 := g(fac2);
desc3 := g(fac3);

desc1n := desc1/Coefficients(desc1)[1];
desc2n := desc2/Coefficients(desc2)[1];
desc3n := desc3/Coefficients(desc3)[1];

desc := desc1n*desc2n*desc3n;
descn := desc/Coefficients(desc)[1];
descn;


